@@ -79,6 +79,10 @@ KNOWN_EXTENSIONS = (
     'tag',
     'trunk',
     'trunk-details',
+    # Add here list of extensions with pointers to the project repo, e.g.
+    # 'bgp',  # http://git.openstack.org/cgit/openstack/neutron-dynamic-routing
+    'fw',  # http://git.openstack.org/cgit/openstack/neutron-fwaas
+    'fwaas',  # http://git.openstack.org/cgit/openstack/neutron-fwaas
 )
 
 # The following is a short reference for understanding attribute info:
