@@ -17,11 +17,12 @@ import functools
 import inspect
 import netaddr
 from oslo_log import log as logging
+from oslo_utils import netutils
+from oslo_utils import strutils
 from oslo_utils import uuidutils
 import six
 
 from neutron_lib._i18n import _
-from neutron_lib.api import converters
 from neutron_lib import constants
 from neutron_lib import exceptions as n_exc
 
@@ -226,8 +227,8 @@ def validate_boolean(data, valid_values=None):
     human readable message indicating why data is invalid.
     """
     try:
-        converters.convert_to_boolean(data)
-    except n_exc.InvalidInput:
+        strutils.bool_from_string(data, strict=True)
+    except ValueError:
         msg = _("'%s' is not a valid boolean value") % data
         LOG.debug(msg)
         return msg
@@ -540,6 +541,22 @@ def validate_ip_address_or_none(data, valid_values=None):
         return validate_ip_address(data, valid_values)
 
 
+def validate_ip_or_subnet_or_none(data, valid_values=None):
+    """Validate data is an IP address, a valid IP subnet string, or None.
+
+    :param data: The data to validate.
+    :param valid_values: Not used!
+    :return: None if data is None or a valid IP address or a valid IP subnet,
+    otherwise a human readable message indicating why the data is neither an
+    IP address nor IP subnet.
+    """
+    msg_ip = validate_ip_address_or_none(data)
+    msg_subnet = validate_subnet_or_none(data)
+    if msg_ip is not None and msg_subnet is not None:
+        return _("'%(data)s' is neither a valid IP address, nor "
+                 "is it a valid IP subnet") % {'data': data}
+
+
 def validate_subnet(data, valid_values=None):
     """Validate data is an IP network subnet string.
 
@@ -700,7 +717,7 @@ def _validate_dict_item(key, key_validator, data):
     # TODO(salv-orlando): Structure of dict attributes should be improved
     # to avoid iterating over items
     val_func = val_params = None
-    for (k, v) in six.iteritems(key_validator):
+    for (k, v) in key_validator.items():
         if k.startswith('type:'):
             # ask forgiveness, not permission
             try:
@@ -735,7 +752,7 @@ def validate_dict(data, key_specs=None):
         return
 
     # Check whether all required keys are present
-    required_keys = [key for key, spec in six.iteritems(key_specs)
+    required_keys = [key for key, spec in key_specs.items()
                      if spec.get('required')]
 
     if required_keys:
@@ -752,7 +769,7 @@ def validate_dict(data, key_specs=None):
 
     # Perform validation and conversion of all values
     # according to the specifications.
-    for key, key_validator in [(k, v) for k, v in six.iteritems(key_specs)
+    for key, key_validator in [(k, v) for k, v in key_specs.items()
                                if k in data]:
         msg = _validate_dict_item(key, key_validator, data)
         if msg:
@@ -822,6 +839,42 @@ def validate_non_negative(data, valid_values=None):
         return msg
 
 
+def validate_port_range_or_none(data, valid_values=None):
+    """Validate data is a range of TCP/UDP port numbers
+
+    :param data: The data to validate
+    :param valid_values: Not used!
+    :returns: None if data is an int between 0 and 65535, or two ints between 0
+    and 65535 with a colon between them, otherwise a human readable message as
+    to why data is invalid.
+    """
+    if data is None:
+        return
+    if validate_string_or_none(data):
+        msg = _("Port range must be a string.")
+        LOG.debug(msg)
+        return msg
+    ports = data.split(':')
+    if len(ports) > 2:
+        msg = _("Port range must be two integers separated by a colon.")
+        LOG.debug(msg)
+        return msg
+    for p in ports:
+        if len(p) == 0:
+            msg = _("Port range must be two integers separated by a colon.")
+            LOG.debug(msg)
+            return msg
+        if not netutils.is_valid_port(p):
+            msg = _("Invalid port: %s.") % p
+            LOG.debug(msg)
+            return msg
+    if len(ports) > 1 and ports[0] > ports[1]:
+        msg = _("First port in a port range must be lower than the second "
+                "port.")
+        LOG.debug(msg)
+        return msg
+
+
 def validate_subports(data, valid_values=None):
     """Validate data is a list of subnet port dicts.
 
@@ -888,11 +941,13 @@ validators = {'type:dict': validate_dict,
               'type:hostroutes': validate_hostroutes,
               'type:ip_address': validate_ip_address,
               'type:ip_address_or_none': validate_ip_address_or_none,
+              'type:ip_or_subnet_or_none': validate_ip_or_subnet_or_none,
               'type:ip_pools': validate_ip_pools,
               'type:mac_address': validate_mac_address,
               'type:mac_address_or_none': validate_mac_address_or_none,
               'type:nameservers': validate_nameservers,
               'type:non_negative': validate_non_negative,
+              'type:port_range': validate_port_range_or_none,
               'type:range': validate_range,
               'type:regex': validate_regex,
               'type:regex_or_none': validate_regex_or_none,
